;
; SYS_SIZE is the number of clicks (16 bytes) to be loaded.
; 0x3000 is 0x30000 bytes = 196kB, more than enough for current
; versions of linux
;
SYSSIZE = 0x3000
;
;	bootsect.s		(C) 1991 Linus Torvalds
;
; bootsect.s is loaded at 0x7c00 by the bios-startup routines, and moves
; iself out of the way to address 0x90000, and jumps there.
;
; It then loads 'setup' directly after itself (0x90200), and the system
; at 0x10000, using BIOS interrupts. 
;
; NOTE! currently system is at most 8*65536 (= 8*64KB = 512KB) bytes long. This should be no
; problem, even in the future. I want to keep it simple. This 512 kB
; kernel size should be enough, especially as this doesn't contain the
; buffer cache as in minix
;
; The loader has been made as simple as possible, and continuos
; read errors will result in a unbreakable loop. Reboot by hand. It
; loads pretty fast by getting whole sectors at a time whenever possible.

.globl begtext, begdata, begbss, endtext, enddata, endbss
.text
begtext:
.data
begdata:
.bss
begbss:
.text

SETUPLEN = 4				; nr of setup-sectors
BOOTSEG  = 0x07c0			; original address of boot-sector
INITSEG  = 0x9000			; we move boot here - out of the way
SETUPSEG = 0x9020			; setup starts here
SYSSEG   = 0x1000			; system loaded at 0x10000 (65536).
ENDSEG   = SYSSEG + SYSSIZE		; where to stop loading

; ROOT_DEV:	0x000 - same type of floppy as boot.
;		0x301 - first partition on first drive etc
ROOT_DEV = 0x306

entry start

; first we are going to move the (this file) "bootsect" code from  
; 0x07C0:0x0000 = 0x7C00 (DS:SI) to 0x9000:0x0000 = 0x90000 (ES:DI)
 
start:
	mov	ax,#BOOTSEG			; load source address 
	mov	ds,ax				; x07C0:0x0000 = 0x7C00 (DS:SI)
	mov	ax,#INITSEG			; load destination address 0x9000
	mov	es,ax				; 0x9000:0x0000 = 0x90000 (ES:DI)
	mov	cx,#256				; specify the counter value 256 (transfer rate is word - 2 bytes) 256*2=512 bytes
	sub	si,si				; set DS segment offset as 0x0000
	sub	di,di				; set ES segment offset as 0x0000
	rep						; repeat the below command @256 times
	movw					; move word (2 bytes)
	jmpi	go,INITSEG		; the CPU execution moved to the 0x9000.

; configure the new address 0x9000 to DS, ES and SS.
; and set stack pointer.
go:	mov	ax,cs
	mov	ds,ax
	mov	es,ax
; put stack at 0x9ff00.
	mov	ss,ax
	mov	sp,#0xFF00			; arbitrary value >>512

; load the setup-sectors directly after the bootblock. at 0x9200
; Note that 'es' is already set up.

load_setup:
	mov	dx,#0x0000			; drive 0, head 0
	mov	cx,#0x0002			; sector 2, track 0
	mov	bx,#0x0200			; address = 512, in INITSEG
	mov	ax,#0x0200+SETUPLEN	; service 2, nr of sectors
	int	0x13				; read it
	jnc	ok_load_setup		; ok - continue
	mov	dx,#0x0000
	mov	ax,#0x0000			; reset the diskette
	int	0x13
	j	load_setup			; repeat the loading 

ok_load_setup:

; Get disk drive parameters, specifically nr of sectors per track

	mov	dl,#0x00			; set drive number = 0
	mov	ax,#0x0800			; AH=8 is get drive parameters
	int	0x13			
	mov	ch,#0x00			; nr of sectors per track returned on CL, so make CH = 0
	seg cs
	mov	sectors,cx			; save CL value (nr of sectors) in to a variable.
	
	;-- I don't know, how this ES segment value changed after call of int 0x13 --
	mov	ax,#INITSEG			; The es was changed when call int 0x13.
	mov	es,ax				; here just re-set it to 0x9000

; Print some inane message

	; set video mode
	; AL = 03h - text mode. 80x25. 16 colors. 8 pages.
	mov al, #0x03
	mov ah, #0x0
	int 0x10

	; Set text-mode cursor shape
	; Show standard blinking text cursor
    mov ch, #6
    mov cl, #7
    mov ah, #1
    int 0x10
    
;INT 10h / AH = 03h - get cursor position and size.
;input:
;BH = page number.
;return:
;DH = row.
;DL = column.
;CH = cursor start line.
;CL = cursor bottom line.

	mov	ah,#0x03			; read cursor pos
	xor	bh,bh
	int	0x10				; it will update current column, row information at DL & DH registers.
	
;INT 10h / AH = 13h - write string.
;input:
;AL = write mode:
;    bit 0: update cursor after writing;
;    bit 1: string contains attributes.
;BH = page number.
;BL = attribute if string contains only characters (bit 1 of AL is zero).
;CX = number of characters in string (attributes are not counted).
;DL,DH = column, row at which to start writing.
;ES:BP points to string to be printed.

	mov	cx,#27				; string length of #msg1
	mov	bx,#0x0007			; page 0, attribute 7 (normal) 
							; character attribute is 8 bit value, low 4 bits forecolor (light gray = 7), 
							; high 4 bits set background color (black = 0)
	mov	bp,#msg1			; the string present in code segment, the same updated in to ES = 0x9000
							; now ES:BX points the string printed.
	mov	ax,#0x1301			; AH = 0x13 write string, 
							; AL = 0x01 string contains attributes.
	int	0x10

	mov ax, #0x8600
	int 0x15
	
;-- once again print, using my own print function
	push msg1
	call p_utils_print_string

	mov ah, #0x86
	int 0x15
	
	
; ok, we've written the message, now
; we want to load the system (at 0x10000)

	mov	ax,#SYSSEG			; SYSSEG is 0x1000
	mov	es,ax				; now load with SYSSEG value 0x1000, it is a 
							; parameter to the function "read_it"
	call	read_it
	call	kill_motor

; After that we check which root-device to use. If the device is
; defined (;= 0), nothing is done and the given device is used.
; Otherwise, either /dev/PS0 (2,28) or /dev/at0 (2,8), depending
; on the number of sectors that the BIOS reports currently.

	seg cs
	mov	ax,root_dev
	cmp	ax,#0
	jne	root_defined
	seg cs
	mov	bx,sectors
	mov	ax,#0x0208		; /dev/ps0 - 1.2Mb
	cmp	bx,#15
	je	root_defined
	mov	ax,#0x021c		; /dev/PS0 - 1.44Mb
	cmp	bx,#18
	je	root_defined
undef_root:
	jmp undef_root
root_defined:
	seg cs
	mov	root_dev,ax

; after that (everyting loaded), we jump to
; the setup-routine loaded directly after
; the bootblock:

	jmpi	0,SETUPSEG

; This routine loads the system at address 0x10000, making sure
; no 64kB boundaries are crossed. We try to load it as fast as
; possible, loading whole tracks whenever we can.
;
; in:	es - starting address segment (normally 0x1000)
;

; RWN - Additional information
; Keep in mind, bootsect.s => this file size is 512 bytes
; setup.s => this file size occupy 4 sectors in disk.
; system image => this image stars at 5th sector in disk,
; so that the below, "sread" value is "5". 
; interrupt 0x13 accepts sector values from 1 to 18 (not 0 to 17)

; One more note: AX reg is used to hold the no. of sectors to be readed.
; Actually for interrupt 0x13 AL reg should hold the value of no. of sectors 
; to be readed.

; Heads, Tracks reading order is listed below:
; head 0 track 0
; head 1 track 0
; head 0 track 1
; head 1 track 1
; etc...,

; The advantage in the above order is, it is more efficient to read both heads(a logical change) 
; at the same track rather than add a physical movement up and have to come back later. 
; This minimises head movement.

;-- declare some variables required for our function --

sread:	.word 1+SETUPLEN	; sectors read of current track
head:	.word 0				; current head
track:	.word 0				; current track

;-- first configure ES:BX [segment:offset] destination data buffer 	--

read_it:
	mov ax,es				; move 0x1000 value in to AX. destination segment base address
	test ax,#0x0fff			; 0x1000 & 0x0fff  (result ZF = 1)
	
die:	jne die				; if(ZF == 0) then die (infinite loop) 
							; [es must be at 64kB boundary]

	xor bx,bx				; bx is segment offset (=0x0)
							; so here we concluded, destination data buffer ES:BX is
							; 0x1000:0x0000 = 0x10000 (final destination address in real mode)

;-- the below is a start of loop, first check whether we reached 	--
;-- the destination address or not (then task is completed) or not	--

rp_read:					; "rp_read" - lable for loop operation
	mov ax,es				; here ES points the destination address, so move that value in to AX
	cmp ax,#ENDSEG			; then compare with end segment value, if it is below then we have not  
							; yet completed the loading process, otherwise we have completed the
							; loading preocess 
							; sample:- [0x1000 and 0x4000 (0x1000 + 0x3000) = not equal, ZF = 0]
	
	jb ok1_read				; (jb = jump if below) jump if first oprend is below second operand
	ret						; in case if first operend is equal or above then second operand then,
							; our job is done, so return.

;-- find out how many bytes are yet to be copied --
;-- total no. of bytes = (total no. of sectors - start of sector no.) * 512 bytes --

ok1_read:					; "ok1_read" -don't panic with this lable, it is just required for a
							; jump from the previous return, OK, lets see 

	seg cs					; current segment is CS (?)
	mov ax,sectors			; load no. of sectors per track [of floppy drive] in to AX (assume 18)
	sub ax,sread			; AX = total no. of sectors - sectors of "bootsetc" + "setup" (18-5 =13)
	mov cx,ax				; remaining sectors value loaded in to CX
	shl cx,#9				; CX = number of remaining sectors * 512 bytes ( << 9) = total no of bytes required to copy (6656)
	add cx,bx				; 

	;-- no. of bytes to copy is calculated, now call "ok2_read" to start read --	
	jnc ok2_read			
	je ok2_read				; if the number of bytes not exceeded 64KB then go to "ok2_read"
							; if ZF = 1 then jump, in what situation it will be usefull? i dnt know.							

	;-- in the current ES segment, 0x1000, 0x2000 or 0x3000, 
	;-- BX contains some offset value, using that value, find out 
	;-- no. of sectors already readed from the ES:BX (offset =0 to curent BX value) --
	xor ax,ax				; if the number of bytes exceeding 64KB then 
	sub ax,bx
	shr ax,#9				; Total number of sectors readed in the current ES segment is 
							; calculated (by number of bytes / 512).

	;-- start reading sectors --
ok2_read:
	call read_track			; read sectors

	;-- check whether all sectors are read out from the current track --
	mov cx,ax				; after "read_track" call, pop AX restored the previous value of AX
							; AX contains the value of no of sectors readed.
	add ax,sread			; "sread" conatins the starting offset of sector number
							; so, add with no. of sectors sucessfully transferred.
							
	seg cs
	
	;-- here we conclude, whether all sectors in a track are moved or not
	;-- if, still some sectors are yet to move then, jump to "ok3_read"
	;-- otherwise continue... --
	cmp ax,sectors			; if both are equal set ZF = 1, otherwise ZF = 0.
	jne ok3_read			; if ZF = 0 then jump. (still some sectors are yet to read)

	;-- all sectors are in a track are moved sucessfully, 
	;-- so go to next track/cylinder --
	mov ax,#1
	sub ax,head				; if the result is zero (means AX & head are having same value),
							; then ZF = 1, otherwise ZF = 0.
							
	;-- if head value is not 1, then go to "ok4_read"						
	jne ok4_read			; if ZF = 0 then jump. (don't increment track and start read)
	inc track				; if "head" value is one, then increment track and start read.

	;-- update "head" value (=1), and star read. --
ok4_read:
	mov head,ax				; update "head" value
	xor ax,ax

	;-- if the control comes here, means, we are going to do one of the below: 
	;-- 1) the sectors remaining to read, in the previous track & head selected.
	;-- 2) or to read the sectors in the next head (=1) (track=0) of FDD
	;-- 3) or to read the sectors in the next track in head (=1) of FDD
	;-- and also updates the destination address Base:Offset [ES:BX] value --
ok3_read:
	mov sread,ax			; update start offset of sector value (= 0, if after "ok4_read")
							; otherwise (jump from "ok2_read") AX contains the number of 
							; sectors readed so far.
	shl cx,#9				; CX contains the no. of sectors sucessfully transferred
							; in the previous read (0x13 int), here we multiply with
							; value 512 to find out the no. of byte are readed in last
							; interrrupt. 
							
	;-- increment the destination address Base:Offset [ES:BX] value --	
	add bx,cx				; increment the offset value with the no. of bytes readed
							; in the last interrupt (0x13 int)
	jnc rp_read				; go back to start of the loop, 
							; if the BX + CX result "not has carry"
							
	;-- if the BX + CX result "has carry", then update "ES" segment value.
	mov ax,es				; load AX with "ES" segment value
	add ax,#0x1000			; add AX with 0x1000 
	mov es,ax				; updated AX value move back to as "ES" segment value.
	xor bx,bx				; reset BX offset value.
	jmp rp_read				; go back to start of the loop

read_track:

;INT 13h / AH = 02h - read disk sectors into memory.
;INT 13h / AH = 03h - write disk sectors.

;input:
;AL = number of sectors to read/write (must be nonzero)
;CH = cylinder number (0..79).
;CL = sector number (1..18).
;DH = head number (0..1).
;DL = drive number (0..3 , for the emulator it depends on quantity of FLOPPY_ files).
;ES:BX points to data buffer.

;return:
;CF set on error.
;CF clear if successful.
;AH = status (0 - if successful).
;AL = number of sectors transferred


	;-- backup general purpose registers
	;-- it is required, in case, FDD read failed, 
	;-- then we can restore these values in "bad_rt" lable
	;-- and repeat the function call "read_back" to read again --
	push ax
	push bx
	push cx
	push dx

	;-- load track/cylinder number --
	mov dx,track			; update track number in to DX, later
							; we update the track number in to CH

	;-- update starting of sector number --						
	mov cx,sread			; update start of sector number
	inc cx					; increment & update sector number in CL

	;-- update track/cylinder number --	
	mov ch,dl				; update track/cylinder number in CH from DX (=0) 
							; keep in mind, for floppy, track and cylinder numbers are 
							; always same for interupt 0x13, we have to pass cylinder 
							; number, cylinder means both up & bottom side of track 
							; for better understand, use the below link 
							; [http://www.jegsworks.com/Lessons/lesson6/lesson6-3.htm]

	;-- update head number --
	mov dx,head				; update head number (=0), here "head" is a 2 byte value, so first update in to DX
	mov dh,dl				; then update DH for head number.
	
	;-- update drive number --
	mov dl,#0				; update drive number (=0) always 0, since it is primary drive for boot.
	and dx,#0x0100			; head number should not be more than 1, and drive number should be 0,
							; so, this instruction just makes sure the above conditions.
							
	;-- update AH for error check --						
	mov ah,#2				; AH = 02h - read disk sectors into memory (AH = 03h - write disk sectors)
							; we are moving the 0x02, it should become zero after read.
	int 0x13				; trigger drive to read by interrupt.
	jc bad_rt				; if CF = 1 then error on reading, go to handle error at "bad_rt"

	;-- backup restore purpose registers --
	pop dx
	pop cx
	pop bx
	pop ax
	
	ret						; return function end
	
bad_rt:	
	mov ax,#0				; AH = 0, then request to reset disk system
	mov dx,#0				; DH = 0, DL = 0, heade & drive number are zero.
	int 0x13				; call interrupt to reset disk system.

	;-- backup restore purpose registers --
	pop dx
	pop cx
	pop bx
	pop ax
	
	jmp read_track			; reset done, go back to read again.

;
; This procedure turns off the floppy drive motor, so
; that we enter the kernel in a known state, and
; don't have to worry about it later.
;
kill_motor:
	push dx
	mov dx,#0x3f2
	mov al,#0
	outb
	pop dx
	ret

;------------------------------------------------------------------------------
;PROCEDURE NAME	:	"p_utils_print_string"
;INPUT			:	load 0 terminated string address in to STACK, then call 
;				:	this function
;WARNINGS		:	none
;AUTHOR NAME	:	rajeshwaran paulchamy
;NOTES			:	
;------------------------------------------------------------------------------
p_utils_print_string:

	;-- save stack pointer --
	pusha				; save all register values in stack
	mov ebp, esp		; move ESP reg value in to EBP reg	

	;-- move first param (at offset zero) to register SI --
	;-- function ptr (2 bytes) + all general purpose registers size (16/32 bytes) --
	mov v_local, #2
	mov al, #8
	mul v_local
	add v_local, #2
	add v_local, ebp
	mov ax, #v_local
	;mov ax, #msg1
	mov si, ax
	;mov si, (ebp + v_ptr_size + (v_ptr_size * 8) + 0)

	cld									; clear direction flag (set DF = 0) to increment SI register

	l1:
		lodsb							; load [SI] contents in to AL register and increment SI = SI + 1

		cmp al, 0						; compare 0 (null) is received
		je l2

		call p_itr_output_teletype

		jmp l1							; continue to lable 'lbl_1'

	l2:

	;-- restore stack pointer --
	mov esp, ebp						; restore ESP reg value from EBP reg
	popa								; restore all register values from stack

ret										; procedure end

;------------------------------------------------------------------------------
;PROCEDURE NAME	:	"p_itr_output_teletype"
;INPUT			:	load a character value in to AL register
;WARNINGS		:	none
;AUTHOR NAME	:	rajeshwaran paulchamy
;NOTES			:
;------------------------------------------------------------------------------
p_itr_output_teletype:

	push ax								; save the register values which are used in this proc

	mov ah, 0x0E						; set interrupt sub function
										; AH=0EH (Teletype output) displays a character on the screen,
										; advancing the cursor and scrolling the screen as necessary.
										; the printing is always done to current active page.

	int 0x10								; Use 10h interrupt to print the character

	pop ax								; restore the register values which are used in this proc
ret

;------------------------------------------------------------------------------
;PROCEDURE NAME	:	"p_utils_print_register16"
;PREREQUISITES	:	load register value in to SI register
;WARNINGS		:	none
;AUTHOR NAME	:	rajeshwaran paulchamy
;NOTES			:
;------------------------------------------------------------------------------
;p_utils_print_register16:

;	m_common_fn_enter					; save stack pointer

;	mov si, m_common_get_param(0)		; move first param (at offset zero) to register SI

;	mov cx, 4h							; repeat iteration 4 times

	; move SI register's most significant nibble to DI register
;	.l1:
;		mov di, si						; move the SI content to DI
;		and di, 0xF000					; do AND operation for the most significant nibble and store it in to DI register
;		shr di, 12						; do right shift of 12 bits to take the most significant nibble to print
;		shl si, 4						; do left shift of 4 bits to remove the most significant nibble of SI register, since that nibble is moved in to DI reg

		; compare DI register if it is less than 0xA then go to lable .l2

;		cmp di, 0xA						; compare DI register and 0x000A
;		jb .l2							; jump to lable .l2

		; if DI reg value is above 0xA then do the below

;		sub di, 0xA						; subtract 0xA from DI reg, if DI has value 0xA then it will become zero
;		add di, 'A'						; now add ASCII value of 'A' to DI reg
;		mov ax, di						; move the modified DI reg value to AX reg
;		jmp .l3							; jump to lable .l3

		; if DI reg value is below 0xA then do the below

;	.l2:
;		add di, '0'						; now add ASCII value of '0' to DI reg
;		mov ax, di						; move the modified DI reg value to AX reg
;		jmp .l3							; jump to lable .l3

		; print the hex value

;	.l3:
;		call p_itr_output_teletype		; print the value in AL reg

		; loop condition checking

;	.l4:
;		dec cx
;		cmp cx, 0
;		je .l5

;		jmp .l1							; continue to lable '.l1'

		; do loop end & function end things

;	.l5:
;		mov al, 'h'
;		call p_itr_output_teletype

;	m_common_fn_leave					; restore stack pointer

;ret										; procedure end

;-----------------------------------------------------------------
;print:
;  buff db 15 dup(0)
;  mov ax,[esp+4]
;  mov bx,0
;itoa_w1:

;  mov cx, ax
; mod cx,10
;  add cx,30h;'0'
; div ax,10
;  mov buff[bx],cl
;  cmp ax,0
;  jnz itoa_w1

;itoa_w2:
;  push buff[bx]
;  call putchar
;  pop  ax
;  cmp  bx,0
;  jnz itoa_w2

;ret
;-----------------------------------------------------------------


;---------------------------------------------------------

v_local:
	.word 0
	
v_ptr_size:
	.word 2
	
sectors:
	.word 0

msg1:
	.byte 13,10
	.ascii "Loading y4 system ..."
	.byte 13,10,13,10,0

.org 508
root_dev:
	.word ROOT_DEV
boot_flag:
	.word 0xAA55

.text
endtext:
.data
enddata:
.bss
endbss:
